@@ -12,7 +12,7 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 69102 2006-07-12 13:10:28Z shh $
+$Id: ZCatalog.py 81976 2007-11-24 13:33:19Z hannosch $
 """
 
 from warnings import warn
@@ -177,6 +177,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         self._v_total = 0
 
         self._catalog = Catalog()
+        self._migrated_280 = True
 
     def __len__(self):
         # Perform a migration of _catalog.__len__ to _catalog._length
@@ -221,7 +222,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             for url in urls:
                 obj = self.resolve_path(url)
-                if not obj and hasattr(self, 'REQUEST'):
+                if obj is None and hasattr(self, 'REQUEST'):
                     obj = self.resolve_url(url, REQUEST)
                 if obj is not None:
                     self.catalog_object(obj, url)
@@ -289,7 +290,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
             p = paths[i]
             obj = self.resolve_path(p)
-            if not obj:
+            if obj is None:
                 obj = self.resolve_url(p, self.REQUEST)
             if obj is not None:
                 try:
@@ -587,12 +588,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     def getobject(self, rid, REQUEST=None):
         """Return a cataloged object given a 'data_record_id_'
         """
-        obj = self.aq_parent.unrestrictedTraverse(self.getpath(rid))
-        if not obj:
-            if REQUEST is None:
-                REQUEST=self.REQUEST
-            obj = self.resolve_url(self.getpath(rid), REQUEST)
-        return obj
+        return self.aq_parent.unrestrictedTraverse(self.getpath(rid))
 
     def getMetadataForUID(self, uid):
         """return the correct metadata given the uid, usually the path"""
